"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const util_1 = require("util");
function oneOf(expected, thing) {
    assert_1.default(typeof thing === 'string', '`thing` has to be of type string');
    if (Array.isArray(expected)) {
        const len = expected.length;
        assert_1.default(len > 0, 'At least one expected value needs to be specified');
        expected = expected.map((i) => String(i));
        if (len > 2) {
            return `one of ${thing} ${expected.slice(0, len - 1).join(', ')}, or ` +
                expected[len - 1];
        }
        else if (len === 2) {
            return `one of ${thing} ${expected[0]} or ${expected[1]}`;
        }
        else {
            return `of ${thing} ${expected[0]}`;
        }
    }
    else {
        return `of ${thing} ${String(expected)}`;
    }
}
function makeNodeErrorWithCode(Base, key, messageFunc) {
    return class NodeError extends Base {
        constructor(...args) {
            super();
            const message = messageFunc(...args);
            Object.defineProperty(this, 'message', {
                value: message,
                enumerable: false,
                writable: true,
                configurable: true
            });
        }
        get code() {
            return key;
        }
        set code(value) {
            Object.defineProperty(this, 'code', {
                configurable: true,
                enumerable: true,
                value,
                writable: true
            });
        }
        toString() {
            return `${this.name} [${key}]: ${this.message}`;
        }
    };
}
function E(sym, val, def, ...otherClasses) {
    def = makeNodeErrorWithCode(def, sym, val);
    if (otherClasses.length !== 0) {
        otherClasses.forEach((cls) => {
            def[cls.name] = makeNodeErrorWithCode(cls, sym, val);
        });
    }
    return def;
}
exports.ERR_INVALID_ARG_VALUE = E('ERR_INVALID_ARG_VALUE', (name, value, reason = 'is invalid') => {
    let inspected = util_1.inspect(value);
    if (inspected.length > 128) {
        inspected = `${inspected.slice(0, 128)}...`;
    }
    return `The argument '${name}' ${reason}. Received ${inspected}`;
}, TypeError, RangeError);
exports.ERR_INVALID_ARG_TYPE = E('ERR_INVALID_ARG_TYPE', (name, expected, actual) => {
    assert_1.default(typeof name === 'string', "'name' must be a string");
    // determiner: 'must be' or 'must not be'
    let determiner;
    if (typeof expected === 'string' && expected.startsWith('not ')) {
        determiner = 'must not be';
        expected = expected.replace(/^not /, '');
    }
    else {
        determiner = 'must be';
    }
    let msg;
    if (name.endsWith(' argument')) {
        // For cases like 'first argument'
        msg = `The ${name} ${determiner} ${oneOf(expected, 'type')}`;
    }
    else {
        const type = name.includes('.') ? 'property' : 'argument';
        msg = `The "${name}" ${type} ${determiner} ${oneOf(expected, 'type')}`;
    }
    // TODO(BridgeAR): Improve the output by showing `null` and similar.
    msg += `. Received type ${typeof actual}`;
    return msg;
}, TypeError);
exports.ERR_INVALID_URL_SCHEME = E('ERR_INVALID_URL_SCHEME', (expected) => `The URL must be ${oneOf(expected, 'scheme')}`, TypeError);
exports.ERR_INVALID_FILE_URL_PATH = E('ERR_INVALID_FILE_URL_PATH', (actual) => `File URL path ${actual}`, TypeError);
exports.ERR_INVALID_FILE_URL_HOST = E('ERR_INVALID_FILE_URL_HOST', (platform) => `File URL host must be "localhost" or empty on ${platform}`, TypeError);
exports.ERR_INVALID_OPT_VALUE = E('ERR_INVALID_OPT_VALUE', (name, value) => `The value "${String(value)}" is invalid for options "${name}"`, TypeError, RangeError);
